package arzumify.polyenergy.api;

// Provides energy to EnergyReceivers. Can be implemented by anything able to tick and provide energy!
public interface EnergyProvider extends EnergyBlock {
    /**
     * When called by a receiver, attempts to remove energy from the provider.
     * Do not attempt to extract energy until {@link EnergyReceiver#ready(EnergyProvider)} is called by the receiver.
     * The receiver may attempt to remove itself from the provider if it is no longer able to receive energy; this will be done via unready.
     *
     * @param amount   The target amount of energy to extract
     * @param receiver The receiver that is attempting to extract energy
     * @return The amount of energy actually extracted.
     */
    long extract(long amount, EnergyReceiver receiver);

    /**
     * When called by a receiver, indicates that the receiver exists and is capable of starting a connection with the provider.
     * This is a good time to verify that the receiver falls under the criteria for your energy provider.
     * We recommend using a Set to store the receivers that are ready to receive energy, such that when your block is no longer
     * able to provide energy, you can call {@link EnergyReceiver#unready(EnergyProvider)} on all of them.
     * Call {@link EnergyReceiver#ready(EnergyProvider)} on the receiver to establish the connection.
     *
     * @param receiver         The receiver that exists.
     */
    void ready(EnergyReceiver receiver);

    /**
     * When called by a receiver, indicates that the receiver that no longer exists or is no longer able to receive energy.
     * You do not need to call {@link EnergyReceiver#unready(EnergyProvider)} on the receiver.
     *
     * @param receiver The receiver that no longer exists.
     */
    void unready(EnergyReceiver receiver);
}